#! /bin/sh

BASE=`pwd -W`
BASE_MSYS=`pwd`

TOP="$BASE"
BUILDTOP="$BASE/build/BJC_temp"
BUILDTOP_MSYS="$BASE_MSYS/build/BJC_temp"
INSTALLTOP="$BASE/build/BJC_install"

CXX="libtool --tag=CXX --mode=compile g++ -g -I$TOP/src/include"
LINK="libtool --tag=CXX --mode=link g++"
INSTALL="libtool --mode=install install"

SUFEXE=".exe"

# Need to run ltdl configure script before using this.
if ! test -d $BUILDTOP; then
   mkdir -p $BUILDTOP
fi

# Create libltdl from the source.
bjc_cwd=`pwd`

cd $TOP
bjc_srcdir=`pwd`

cd $BUILDTOP
bjc_destdir=`pwd`

mkdir -p $BUILDTOP/libltdl

cd $BUILDTOP/libltdl
$bjc_srcdir/libltdl/configure --prefix=$INSTALLTOP --enable-ltdl-install
result=$?
if test "$result" -ne "0"; then
   exit;
fi

make install
result=$?
if test "$result" -ne "0"; then
   exit;
fi
cd $bjc_cwd


LIBLTDL_CFLAGS="-I$bjc_srcdir/libltdl"
LIBLTDL_LFLAGS=""
LIBLTDL_LIBS="$bjc_destdir/libltdl/libltdl.la"


# Compile and link the static library.
mkdir -p $BUILDTOP/src/libs/Static

echo ""
echo "--- Compiling Static.cpp to Static.lo ---"
$CXX -c $TOP/src/libs/Static/Static.cpp -o $BUILDTOP/src/libs/Static/Static.lo

echo ""
echo "--- Linking Static.lo into libStatic.la ---"
$LINK -o $BUILDTOP/src/libs/Static/libStatic.la $BUILDTOP/src/libs/Static/Static.lo -static


# Compile and link the shared library.
mkdir -p $BUILDTOP/src/libs/Shared
echo ""
echo "--- Compiling Shared.cpp to Shared.lo ---"
$CXX -c $TOP/src/libs/Shared/Shared.cpp -o $BUILDTOP/src/libs/Shared/Shared.lo

echo ""
echo "--- Linking Shared.lo into libShared.la ---"
$LINK -o $BUILDTOP/src/libs/Shared/libShared.la $BUILDTOP/src/libs/Shared/Shared.lo -rpath "$INSTALLTOP/lib"


# Compile and link the plugin.
mkdir -p $BUILDTOP/src/plugins/LTDLOpened
echo ""
echo "--- Compiling LTDLOpened.cpp to LTDLOpened.lo ---"
$CXX -c $TOP/src/plugins/LTDLOpened/LTDLOpened.cpp -o $BUILDTOP/src/plugins/LTDLOpened/LTDLOpened.lo

echo ""
echo "--- Linking LTDLOpened.lo into libLTDLopened.la ---"
$LINK -module -o $BUILDTOP/src/plugins/LTDLOpened/LTDLOpened.la $BUILDTOP/src/plugins/LTDLOpened/LTDLOpened.lo -rpath "$INSTALLTOP/lib"



# Compile and link the application without plugin.
mkdir -p $BUILDTOP/src/apps/libtool_test_np
echo ""
echo "--- Compiling NP App : main.cpp to main.lo ---"
$CXX -c $TOP/src/apps/libtool_test_np/main.cpp -o $BUILDTOP/src/apps/libtool_test_np/main.lo

echo ""
echo "--- Static Linking NP App : main.lo + libShared.la + libStatic.la into libtool_test_np.exe ---"
$LINK -all-static -o $BUILDTOP/src/apps/libtool_test_np/static_libtool_test_np$SUFEXE $BUILDTOP/src/apps/libtool_test_np/main.lo $BUILDTOP/src/libs/Shared/libShared.la $BUILDTOP/src/libs/Static/libStatic.la

echo ""
echo "--- Linking NP App : main.lo + libShared.la + libStatic.la into libtool_test_np.exe ---"
$LINK -o $BUILDTOP/src/apps/libtool_test_np/libtool_test_np$SUFEXE $BUILDTOP/src/apps/libtool_test_np/main.lo $BUILDTOP/src/libs/Shared/libShared.la $BUILDTOP/src/libs/Static/libStatic.la



## Compile and link the application with the plugin.
mkdir -p $BUILDTOP/src/apps/libtool_test
echo ""
echo "--- Compiling App : main.cpp to main.lo ---"
$CXX -c $TOP/src/apps/libtool_test/main.cpp -o $BUILDTOP/src/apps/libtool_test/main.lo $LIBLTDL_CFLAGS

echo ""
echo "--- Static Linking App : main.lo + libShared.la + libStatic.la + libLTDLOpened into static_libtool_test.exe ---"
$LINK -all-static -o $BUILDTOP/src/apps/libtool_test/static_libtool_test$SUFEXE $BUILDTOP/src/apps/libtool_test/main.lo $BUILDTOP/src/libs/Shared/libShared.la $BUILDTOP/src/libs/Static/libStatic.la -dlopen $BUILDTOP/src/plugins/LTDLOpened/LTDLOpened.la $LIBLTDL_LFLAGS $LIBLTDL_LIBS 

echo ""
echo "--- Linking App : Linking main.lo + libShared.la + libStatic.la + libLTDLOpened into libtool_test.exe ---"
$LINK -o $BUILDTOP/src/apps/libtool_test/libtool_test$SUFEXE $BUILDTOP/src/apps/libtool_test/main.lo $BUILDTOP/src/libs/Shared/libShared.la $BUILDTOP/src/libs/Static/libStatic.la -dlopen $BUILDTOP/src/plugins/LTDLOpened/LTDLOpened.la $LIBLTDL_LFLAGS $LIBLTDL_LIBS 



echo ""
echo "--- Installing files ---"
mkdir -p $INSTALLTOP/lib
mkdir -p $INSTALLTOP/bin
$INSTALL -c $BUILDTOP/libltdl/libltdl.la $INSTALLTOP/lib/libltdl.la
$INSTALL -c $BUILDTOP/src/libs/Shared/libShared.la $INSTALLTOP/lib/libShared.la
$INSTALL -c $BUILDTOP/src/plugins/LTDLOpened/LTDLOpened.la $INSTALLTOP/bin/LTDLOpened.la
$INSTALL -c $BUILDTOP/src/apps/libtool_test_np/libtool_test_np$SUFEXE $INSTALLTOP/bin/libtool_test_np$SUFEXE
$INSTALL -c $BUILDTOP/src/apps/libtool_test_np/static_libtool_test_np$SUFEXE $INSTALLTOP/bin/static_libtool_test_np$SUFEXE
$INSTALL -c $BUILDTOP/src/apps/libtool_test/libtool_test$SUFEXE $INSTALLTOP/bin/libtool_test$SUFEXE
$INSTALL -c $BUILDTOP/src/apps/libtool_test/static_libtool_test$SUFEXE $INSTALLTOP/bin/static_libtool_test$SUFEXE
libtool -n --mode=finish $INSTALLTOP/lib

cwd=`pwd`
cd $BUILDTOP

echo ""
echo "--- Executing libtool_test_np.exe using libtool by executing in its build directory ---"
libtool --mode=execute src/apps/libtool_test_np/libtool_test_np$SUFEXE

echo ""
echo "--- Executing static_libtool_test_np.exe using libtool by executing in its build directory ---"
libtool --mode=execute src/apps/libtool_test_np/static_libtool_test_np$SUFEXE

echo ""
echo "--- Executing libtool_test.exe using libtool by executing in its build directory ---"
libtool --mode=execute -dlopen $BUILDTOP/src/plugins/LTDLOpened/LTDLOpened.la src/apps/libtool_test/libtool_test$SUFEXE

echo ""
echo "--- Executing static_libtool_test.exe using libtool by executing in its build directory ---"
libtool --mode=execute -dlopen $BUILDTOP/src/plugins/LTDLOpened/LTDLOpened.la src/apps/libtool_test/static_libtool_test$SUFEXE



# Try executing the installed programs
cd $INSTALLTOP/bin

echo ""
echo "--- Executing static_libtool_test_np.exe using libtool by executing in its build directory ---"
./static_libtool_test_np$SUFEXE

echo ""
echo "--- Executing static_libtool_test.exe using libtool by executing in its build directory ---"
./static_libtool_test$SUFEXE


export LD_LIBRARY_PATH="$LD_LIBRARY_PATH:$INSTALLTOP/lib"

echo ""
echo "--- Executing libtool_test_np.exe using libtool by executing in its build directory ---"
./libtool_test_np$SUFEXE

echo ""
echo "--- Executing libtool_test.exe using libtool by executing in its build directory ---"
./libtool_test$SUFEXE

cd $bjc_cwd

